#define M_PI 3.1415926535897932384626433832795

varying vec3 e;
varying vec3 n;

uniform float elapsedTime;

uniform float temperature;

uniform sampler2D coronaTexture;
uniform sampler2D temperatureTexture;

uniform float coronaFalloff;
uniform float coronaRadius;
uniform float coronaAlpha;
uniform float coronaEdgeBlend;
uniform float coronaTexCurve;

uniform float cmeRadius;
uniform float cmeGlowPower;

uniform float moveSpeed;
uniform float stretch;

varying vec3 vPosition;
varying vec3 vNormal;
varying vec2 vUv;

varying float vBlast;
uniform vec3 blaster;

vec3 col1 = vec3(0.45, 0.0, 0.25);
vec3 col2 = vec3(0.75, 0.0, 0.04);
vec3 col3 = vec3(1.0, 0.73, 0.0);
vec3 col4 = vec3(1.0, 1.0, 1.0);

float stop1 = 0.0;
float stop2 = 0.2;
float stop3 = 0.9;
float stop4 = 1.0;

float cRadius;

vec3 gradientLookup(float loc);
vec3 warp(vec3 offset, float animationSpeed);

float rand(vec2 co){
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

float lerp(float edge0, float edge1, float value) {
    return (clamp(value, edge0, edge1) - edge0) / (edge1 - edge0);
}

vec4 get(float uSpeed, float vSpeed) {

	float bias = 0.0;

	float radius = length(vPosition);
	float r = lerp(1.0+bias, cRadius+bias, radius+bias);

	float desyncV = sin(M_PI * 2.0 * normalize(vPosition).x);
	float desyncU = sin(M_PI * 2.0 * normalize(vPosition).z);
	desyncV = (desyncV+1.0) * 0.5;
	desyncU = (desyncU+1.0) * 0.5;

	r = pow(r, 1.0/coronaTexCurve) + elapsedTime * vSpeed + desyncV;
	return texture2D(coronaTexture, vec2(vUv.x+elapsedTime*uSpeed+desyncU, r));
}

vec3 gradientLookup(float loc) {
    vec3 color = vec3(0,0,0);
    color = mix(col1,  col2, smoothstep(stop1, stop2, loc));
    color = mix(color, col3, smoothstep(stop2, stop3, loc));
    color = mix(color, col4, smoothstep(stop3, stop4, loc));
    return color;
}

void main() {
	cRadius = coronaRadius + vBlast * cmeRadius;
	float power = abs(pow(dot(-e, n), coronaFalloff) * coronaAlpha);
	float dist = length(vPosition);
	float innerFade = smoothstep(1.0, coronaEdgeBlend, dist);
	float outerFade = 1.0 - smoothstep(1.0, cRadius, dist);

	float owt = 1.0/((dist-1.0) * 40.0) + mod(elapsedTime, 1.0);
	vec2 lookup = vec2(vUv.x+elapsedTime, owt);
	vec4 textureSample = texture2D(coronaTexture, lookup);

	gl_FragColor = textureSample * vec4(col4, 1.0);
	gl_FragColor.a = power * min(innerFade, outerFade);

	float SPEEEEEED = 0.5;;
	vec4 a = get( 0.14 * SPEEEEEED, -0.1  );
	vec4 b = get(-0.2  * SPEEEEEED, -0.04 );
	gl_FragColor = (a*0.5) + (b*0.5);
	gl_FragColor.a = power * min(innerFade, outerFade);
	

	float intensity = length(gl_FragColor);
	intensity = clamp(intensity + vBlast*cmeGlowPower, 0.0, 1.0);
	intensity = pow(intensity, 2.5);

	gl_FragColor.rgb = texture2D(temperatureTexture, vec2(intensity, temperature)).rgb;
}